@@ -67,13 +67,24 @@ def _prune_unused_defs(schema: dict) -> dict:
         walk(value, current_def=def_name)
 
     # Figure out what defs were referenced directly or recursively
-    def def_is_referenced(def_name):
+    def def_is_referenced(def_name, parent_def_names: set[str] | None = None):
         if def_name in root_defs:
             return True
         references = referenced_by.get(def_name)
         if references:
-            for reference in references:
-                if def_is_referenced(reference):
+            if parent_def_names is None:
+                parent_def_names = set()
+
+            # Handle recursion by excluding references already present in parent references
+            parent_def_names = parent_def_names | {def_name}
+            valid_references = [
+                reference
+                for reference in references
+                if reference not in parent_def_names
+            ]
+
+            for reference in valid_references:
+                if def_is_referenced(reference, parent_def_names):
                     return True
         return False
 
