@@ -384,6 +384,22 @@ async def test_forward_raw_outside_context_raises_error():
         await forward_raw(new_x=1, old_y=2)
 
 
+def test_transform_args_with_parent_defaults():
+    """Test that transform_args with parent defaults works."""
+
+    class CoolModel(BaseModel):
+        x: int = 10
+
+    def parent_tool(cool_model: CoolModel) -> int:
+        return cool_model.x
+
+    tool = Tool.from_function(parent_tool)
+
+    new_tool = Tool.from_tool(tool)
+
+    assert new_tool.parameters["$defs"] == tool.parameters["$defs"]
+
+
 def test_transform_args_validation_unknown_arg(add_tool):
     """Test that transform_args with unknown arguments raises ValueError."""
     with pytest.raises(
@@ -1141,7 +1157,7 @@ class TestTransformToolOutputSchema:
 
         result = await new_tool.run({"x": 3})
         # Should wrap string result
-        assert result.structured_content == {"result": 'Custom: {\n  "value": 3\n}'}
+        assert result.structured_content == {"result": 'Custom: {"value":3}'}
 
     def test_transform_custom_function_fallback_to_parent(self, base_string_tool):
         """Test that custom function without output annotation falls back to parent."""
