@@ -6,7 +6,11 @@ import pytest
 from fastapi import Body, FastAPI, Path, Query
 from pydantic import BaseModel, Field
 
-from fastmcp.utilities.openapi import parse_openapi_to_http_routes
+from fastmcp.utilities.openapi import (
+    _combine_schemas,
+    _replace_ref_with_defs,
+    parse_openapi_to_http_routes,
+)
 
 # --- Test Data: Static OpenAPI Schema Dictionaries --- #
 
@@ -683,6 +687,29 @@ def test_multiple_tags_preserved(bookstore_schema):
     assert len(get_books.tags) == 3
 
 
+def test_openapi_extensions(petstore_schema):
+    """Test that OpenAPI extensions (x-*) are correctly parsed from operations."""
+    # Add extensions to a route
+    petstore_schema["paths"]["/pets"]["get"]["x-rate-limit"] = 100
+    petstore_schema["paths"]["/pets"]["get"]["x-custom-auth"] = "bearer"
+    petstore_schema["paths"]["/pets"]["get"]["x-internal"] = True
+
+    # Parse the modified schema
+    routes = parse_openapi_to_http_routes(petstore_schema)
+
+    # Find the GET /pets route
+    get_pets = next(
+        (r for r in routes if r.method == "GET" and r.path == "/pets"), None
+    )
+    assert get_pets is not None
+
+    # Should have extensions
+    assert get_pets.extensions["x-rate-limit"] == 100
+    assert get_pets.extensions["x-custom-auth"] == "bearer"
+    assert get_pets.extensions["x-internal"] is True
+    assert len(get_pets.extensions) == 3
+
+
 # --- Tests for BookStore schema --- #
 
 
@@ -1023,6 +1050,9 @@ def test_openapi_30_reference_resolution(openapi_30_with_references):
     # or it still has a $ref field
     assert "properties" in category or "$ref" in category
 
+    combined_schema = _combine_schemas(route)
+    assert "#/$defs/" in combined_schema["properties"]["category"]["$ref"]
+
 
 def test_openapi_31_reference_resolution(openapi_31_with_references):
     """Test that references are correctly resolved in OpenAPI 3.1 schemas."""
@@ -1057,6 +1087,9 @@ def test_openapi_31_reference_resolution(openapi_31_with_references):
     # or it still has a $ref field
     assert "properties" in category or "$ref" in category
 
+    combined_schema = _combine_schemas(route)
+    assert "#/$defs/" in combined_schema["properties"]["category"]["$ref"]
+
 
 def test_consistent_output_across_versions(
     openapi_30_with_references, openapi_31_with_references
@@ -1093,3 +1126,103 @@ def test_consistent_output_across_versions(
         "properties"
     ]
     assert set(schema_30.keys()) == set(schema_31.keys())
+
+
+class TestReplaceRefWithDefs:
+    @pytest.fixture(scope="class")
+    def schemas(self):
+        """Provide test schemas for _replace_ref_with_defs function."""
+        return {
+            "ref_type": {
+                "$ref": "#/components/schemas/RefFoo",
+            },
+            "object_type": {
+                "type": "object",
+                "properties": {"$ref": "#/components/schemas/ObjectFoo"},
+            },
+            "array_type": {
+                "type": "array",
+                "items": {"$ref": "#/components/schemas/ArrayFoo"},
+            },
+            "any_of_type": {
+                "anyOf": [
+                    {"$ref": "#/components/schemas/AnyOfFoo"},
+                    {"$ref": "#/components/schemas/AnyOfBar"},
+                ]
+            },
+            "all_of_type": {
+                "allOf": [
+                    {"$ref": "#/components/schemas/AllOfFoo"},
+                    {"$ref": "#/components/schemas/AllOfBar"},
+                ]
+            },
+            "one_of_type": {
+                "oneOf": [
+                    {"$ref": "#/components/schemas/OneOfFoo"},
+                    {"$ref": "#/components/schemas/OneOfBar"},
+                ]
+            },
+            "nested_type": {
+                "type": "object",
+                "properties": {
+                    "pets": {
+                        "oneOf": [
+                            {"$ref": "#/components/schemas/Cat"},
+                            {"$ref": "#/components/schemas/Dog"},
+                        ]
+                    },
+                },
+            },
+        }
+
+    def test_replace_direct_ref(self, schemas):
+        """Test replacing direct $ref references."""
+        result = _replace_ref_with_defs(schemas["ref_type"])
+        assert result == {"$ref": "#/$defs/RefFoo"}
+
+    def test_replace_object_property_ref(self, schemas):
+        """Test replacing $ref in object properties."""
+        result = _replace_ref_with_defs(schemas["object_type"])
+        assert result == {
+            "type": "object",
+            "properties": {"$ref": "#/$defs/ObjectFoo"},
+        }
+
+    def test_replace_array_items_ref(self, schemas):
+        """Test replacing $ref in array items."""
+        result = _replace_ref_with_defs(schemas["array_type"])
+        assert result == {
+            "type": "array",
+            "items": {"$ref": "#/$defs/ArrayFoo"},
+        }
+
+    def test_replace_any_of_refs(self, schemas):
+        """Test replacing $ref in anyOf schemas."""
+        result = _replace_ref_with_defs(schemas["any_of_type"])
+        assert result == {
+            "anyOf": [{"$ref": "#/$defs/AnyOfFoo"}, {"$ref": "#/$defs/AnyOfBar"}]
+        }
+
+    def test_replace_all_of_refs(self, schemas):
+        """Test replacing $ref in allOf schemas."""
+        result = _replace_ref_with_defs(schemas["all_of_type"])
+        assert result == {
+            "allOf": [{"$ref": "#/$defs/AllOfFoo"}, {"$ref": "#/$defs/AllOfBar"}]
+        }
+
+    def test_replace_one_of_refs(self, schemas):
+        """Test replacing $ref in oneOf schemas."""
+        result = _replace_ref_with_defs(schemas["one_of_type"])
+        assert result == {
+            "oneOf": [{"$ref": "#/$defs/OneOfFoo"}, {"$ref": "#/$defs/OneOfBar"}]
+        }
+
+    def test_replace_nested_refs(self, schemas):
+        """Test replacing $ref in deeply nested schema structures."""
+        result = _replace_ref_with_defs(schemas["nested_type"])
+        assert result == {
+            "type": "object",
+            "properties": {
+                "pets": {"oneOf": [{"$ref": "#/$defs/Cat"}, {"$ref": "#/$defs/Dog"}]}
+            },
+        }
